using System.Collections.Generic;
using System;
using UnityEngine;
using UnityEngine.UI;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using System.Linq;
using MVR.FileManagementSecure;
using UnityEngine.EventSystems;

namespace Ruvik
{
    public class CMenu : MVRScript
    {
        string ASSETBUNDLEPATH;
        string ASSETBUNDLENAME = "menu";
        int MAXENTRIES = 50;
        string appearancePath = "Custom/Atom/Person/Appearance", subscenePath = "Custom/SubScene", assetBundlePath = "Custom/Assets";
        List<ShortCut> appearanceShortcuts = null , subsceneShortcuts = null , assetsBundleShortcuts = null;
        bool checkForAdding = false;
        public static  GameObject CMmenu, CMatomEntry;
        RectTransform CMcontent;
        float entryHeight;
        RectTransform atomEntryRect, atomEntryTextRect, atomEntryButtonRect;
        Text atomEntryText;

        public void OnBindingsListRequested(List<object> bindings)
        {
            bindings.Add(new JSONStorableAction("show list", () => ShowMenu(false)));
            bindings.Add(new JSONStorableAction("show list (all)", () => ShowMenu(true)));
            bindings.Add(new JSONStorableAction("add atom", () =>
            {
                checkForAdding = true;
                SuperController.singleton.AddAtomByType("Person", false, false, false);
            }));
            bindings.Add(new JSONStorableAction("add asset", () =>
            {
                checkForAdding = true;
                SuperController.singleton.AddAtomByType("CustomUnityAsset", false, false, false);
            }));
            bindings.Add(new JSONStorableAction("add subscene", () =>
            {
                checkForAdding = true;
                SuperController.singleton.AddAtomByType("SubScene", false, false, false);
            }));
        }
        public void Print(string str)
        {
            SuperController.LogMessage(str);
        }
        public string GetDir(string str)
        {
            var sanitizedStr = str.Contains(".") ? str.Substring(0, str.LastIndexOf('.') - 1) : str;
            char div = sanitizedStr.Contains('\\') ? '\\' : '/';
            var splited = sanitizedStr.Split(div);
            string dir = splited[0];
            for (int i = 1; i < splited.Length - 1; i++)
            {
                dir += "/" + splited[i];
            }
            return dir;
        }
        public void OnAtomAdded(Atom atom)
        {
            if (!checkForAdding) return;
            checkForAdding = false;
            switch (atom.type)
            {
                case "CustomUnityAsset":
                    {
                        var assetManager = atom.GetStorableByID("asset");
                        SuperController.singleton.GetMediaPathDialog((str) =>
                        {
                            if (str == null || str == "")
                            {
                                SuperController.singleton.RemoveAtom(atom);
                                return;
                            }
                            assetManager.SetUrlParamValue("assetUrl", str);
                            assetBundlePath = GetDir(str);
                        }, "assetbundle", assetBundlePath, false, true, false, null, false, assetsBundleShortcuts);
                        break;
                    }
                case "SubScene":
                    {
                        var subSceneManager = atom.GetStorableByID("SubScene");
                        SuperController.singleton.GetMediaPathDialog((str) =>
                        {
                            if (str == null || str == "")
                            {
                                SuperController.singleton.RemoveAtom(atom);
                                return;
                            }
                            foreach (var item in subSceneManager.GetAllParamAndActionNames())
                            {
                                Print(item);
                            }
                            subSceneManager.SetUrlParamValue("browsePath", str);
                            subSceneManager.CallAction("LoadSubSceneWithPath");
                            subscenePath = GetDir(str);
                        }, "json", subscenePath, false, true, false, null, false, subsceneShortcuts);
                    }
                    break;
                case "Person":
                    {
                        atom.ToggleOn();
                        var appearancePresetManager = atom.presetManagerControls.Find(pm => pm.name == "AppearancePresets");
                        SuperController.singleton.GetMediaPathDialog((str) =>
                        {
                            if (str == null || str == "")
                            {
                                SuperController.singleton.RemoveAtom(atom);
                                return;
                            }
                            appearancePresetManager.SetUrlParamValue("presetBrowsePath", str);
                            appearancePresetManager.CallAction("LoadPreset");
                            atom.ToggleOn();
                            appearancePath = GetDir(str);
                        }, "vap", appearancePath, false, true, false, null, false, appearanceShortcuts);
                    }
                    break;
            }
        }
        public override void Init()
        {
            if (GameObject.Find("CM_menu") != null)
                Print("you fucked up");
            SuperController.singleton.BroadcastMessage("OnActionsProviderAvailable", this, SendMessageOptions.DontRequireReceiver);
            SuperController.singleton.onAtomAddedHandlers = OnAtomAdded;
            ASSETBUNDLEPATH = GetPluginPath(this) + "/assets/cm";
            Request request = new Request { path = ASSETBUNDLEPATH, callback = OnAssetBundleLoaded };
            AssetLoader.QueueLoadAssetBundleFromFile(request);
            appearanceShortcuts = FileManagerSecure.GetShortCutsForDirectory(appearancePath, false, false, true, true);
            subsceneShortcuts = FileManagerSecure.GetShortCutsForDirectory(subscenePath, false, false, true, true);
            assetsBundleShortcuts = FileManagerSecure.GetShortCutsForDirectory(assetBundlePath, false, false, true, true);

            CreateButton("toggle menu").button.onClick.AddListener(() => { CMmenu.SetActive(true); });
            CreateButton("add atom").button.onClick.AddListener(() => { checkForAdding = true; SuperController.singleton.AddAtomByType("Person", false, false, false); });
            CreateButton("add asset").button.onClick.AddListener(() => { checkForAdding = true; SuperController.singleton.AddAtomByType("CustomUnityAsset", false, false, false); });
            CreateButton("add subscene").button.onClick.AddListener(() => { checkForAdding = true; SuperController.singleton.AddAtomByType("SubScene", false, false, false); });

        }
        void InitCMUI()
        {
            CMmenu.AddComponent<goAwayMenu>();
            CMcontent = GameObject.Find("CM_content").GetComponent<RectTransform>();
            CMatomEntry = GameObject.Find("CM_entry_rect");
            atomEntryRect = CMatomEntry.GetComponent<RectTransform>();
            entryHeight = atomEntryRect.sizeDelta.y;
            GetAddButton("person").onClick.AddListener(() =>
            {
                CMmenu.SetActive(false);
                checkForAdding = true;
                SuperController.singleton.AddAtomByType("Person", false, false, false);
            });
            GetAddButton("asset").onClick.AddListener(() =>
            {
                CMmenu.SetActive(false);
                checkForAdding = true;
                SuperController.singleton.AddAtomByType("CustomUnityAsset", false, false, false);
            });
            GetAddButton("subscene").onClick.AddListener(() =>
            {
                CMmenu.SetActive(false);
                checkForAdding = true;
                SuperController.singleton.AddAtomByType("SubScene", false, false, false);
            });

            for (int i = 0; i < MAXENTRIES; i++)
            {
                CreateAtomEntry(i);
            }
            CMmenu.SetActive(false);
        }
        Button GetAddButton(string name)
        {
            var btn = GameObject.Find("CM_btn_add_" + name).GetComponent<Button>();
            return btn != null ? btn : GameObject.Find("CM_btn_add_" + name).AddComponent<Button>();
        }
        public static string GetPluginPath(MVRScript self)
        {
            string id = self.name.Substring(0, self.name.IndexOf('_'));
            string filename = self.manager.GetJSON()["plugins"][id].Value;
            return filename.Substring(0, filename.LastIndexOfAny(new char[] { '/', '\\' }));
        }

        public void OnDestroy()
        {
            SuperController.singleton.BroadcastMessage("OnActionsProviderDestroyed", this, SendMessageOptions.DontRequireReceiver);
            AssetLoader.DoneWithAssetBundleFromFile(ASSETBUNDLEPATH);
            Destroy(CMmenu);
        }
        public void OnDisable()
        {
            CMmenu.SetActive(false);
        }
        public void ShowMenu(bool all)
        {
            CMmenu.SetActive(!CMmenu.activeInHierarchy);
            if (CMmenu.activeInHierarchy)
            {
                var atoms = SuperController.singleton.GetAtoms();
                if (!all)
                {
                    atoms = atoms.Where(atom => atom.hidden == false).ToList();
                }
                CMcontent.GetComponent<RectTransform>().sizeDelta = new Vector2(CMcontent.GetComponent<RectTransform>().sizeDelta.x, Math.Max(entryHeight * 10, entryHeight * atoms.Count));
                int bound = Math.Min(atoms.Count, MAXENTRIES);
                for (int i = 0; i < bound; i++)
                {
                    UpdateAtomEntry(atoms[i], i);
                }
                if (atoms.Count < 20)
                    for (int i = atoms.Count; i <= 20; i++)
                        DisableAtomEntry(i);
            }

        }
        void CreateAtomEntry(int index)
        {
            var entry = Instantiate(atomEntryChild);
            CopyRect(entry.GetComponent<RectTransform>(), atomEntryRect);
            entry.GetComponent<RectTransform>().anchoredPosition = new Vector2(0, index * -(entryHeight + 0));
            for (int i = 0; i < entry.transform.childCount; i++)
            {
                entry.transform.GetChild(i).gameObject.AddComponent<EntryComponent>();
            }
        }
        void DisableAtomEntry(int index)
        {
            var entry = CMcontent.GetChild(index + 1);
            entry.gameObject.SetActive(false);
        }
        void UpdateAtomEntry(Atom atom, int index)
        {
            var entry = CMcontent.GetChild(index + 1);

            if (!entry.gameObject.activeInHierarchy)
                entry.gameObject.SetActive(true);

            entry.GetComponentInChildren<Text>().text = atom.name.Replace('\\','>');
            var entrycomps = entry.GetComponentsInChildren<EntryComponent>();
            foreach (var item in entrycomps)
                item.SetAtom(atom);
        }
        void CopyRect(RectTransform a, RectTransform b)
        {
            a.SetParent(b.parent);
            a.localPosition = b.localPosition;
            a.localScale = b.localScale;
            a.sizeDelta = b.sizeDelta;
            a.anchorMax = b.anchorMax;
            a.anchorMin = b.anchorMin;
            a.offsetMax = b.offsetMax;
            a.offsetMin = b.offsetMin;
            a.pivot = b.pivot;
            a.anchoredPosition = b.anchoredPosition;

        }
        GameObject atomEntryChild;
        private void OnAssetBundleLoaded(Request aRequest)
        {
            var menu = aRequest.assetBundle.LoadAsset<GameObject>("assets/cm/cm.prefab");
            var atomEntry = aRequest.assetBundle.LoadAsset<GameObject>("assets/cm/cm_entry.prefab");
            if (menu != null)
            {
                CMmenu = Instantiate(menu);
                atomEntryChild = atomEntry;
                try
                {
                    InitCMUI();
                }
                catch (Exception e)
                {
                    SuperController.LogError(e.Source + e.Message);
                }
            }
            else
            {
                Print("not found");
            }
        }
        public class goAwayMenu : MonoBehaviour, IPointerExitHandler , IPointerEnterHandler
        {
            bool isMoused;
            public void OnPointerEnter(PointerEventData data)
            {
                isMoused = true;
            }
            public void OnPointerExit(PointerEventData data)
            {
                isMoused = false;
            }
            public void Update()
            {
                if (isMoused)
                    return;
                if(Input.GetKeyDown(KeyCode.Mouse0))
                {
                    CMmenu.SetActive(false);
                }
            }
        }
        public class EntryComponent : MonoBehaviour, IPointerDownHandler
        {
            Atom atom;
            public void OnPointerDown(PointerEventData data)
            {
                if (atom == null)
                    return;
                if (this.name != "CM_entry_name")
                    switch (gameObject.name)
                    {
                        case "CM_entry_delete":
                            SuperController.singleton.RemoveAtom(atom);
                            GetComponent<Image>().color = Color.gray;
                            //Menu.CMmenu.SetActive(false);
                            break;
                        case "CM_entry_show":
                            atom.hidden = !atom.hidden;
                            GetComponent<Image>().color = !atom.hidden ? Color.white : Color.gray;
                            break;
                        case "CM_entry_collision":
                            atom.collisionEnabled = !atom.collisionEnabled;
                            GetComponent<Image>().color = atom.collisionEnabled ? Color.white : Color.gray;
                            break;
                        case "CM_entry_state":
                            atom.ToggleOn();
                            GetComponent<Image>().color = atom.on ? Color.white : Color.gray;
                            break;
                    }
                else
                {
                    //left click
                    if (data.pointerId == -1)
                    {
                        SuperController.singleton.SelectController(atom.mainController, false, false, false, true);
                    }
                    //right click
                    else if (data.pointerId == -2)
                    {
                        SuperController.singleton.FocusOnController(atom.mainController);
                    }
                    //find a good use for middle click
                }
            }
            public void SetAtom(Atom atom)
            {
                this.atom = atom;
                switch (gameObject.name)
                {
                    case "CM_entry_delete":
                        GetComponent<Image>().color = Color.white;
                        break;
                    case "CM_entry_show":
                        GetComponent<Image>().color = !atom.hidden ? Color.white : Color.gray;
                        break;
                    case "CM_entry_collision":
                        GetComponent<Image>().color = atom.collisionEnabled ? Color.white : Color.gray;
                        break;
                    case "CM_entry_state":
                        GetComponent<Image>().color = atom.on ? Color.white : Color.gray;
                        break;
                }
            }
        }
    }
}
